/* translating iconic rushHour problems into the corresponding
   symbolic representation

   the iconic representation of a rushHour problem is a 36 element
   array representing a virtual 6 x 6 array of integers, where an
   integer represents a vehicle id and the id is put into each
   grid square where that vehicle would occupy

   the symbolic representation of a rushHour problem is an initial
   state and a goal state, the initial state is composed of the
   static information and the initial fluents.  the current ontology
   that this translator expects is the following.  a vehicle is
   represented by 4 pieces of information: its id, the location of
   the "head" of the vehicle, the vehicle's orientation, and the
   vehicle's length.  the "head" is the upper leftmost part of the
   vehicle.  the ids are integers.

   this means that the only info in the initial state is the static
   info: orientation and length of each vehicle (by id); and the
   fluent info: location of each vehicle's head (by id).

   red car always has id = 1, orientation = horizontal, and length = 2
   goal is that the red car gets to position (4,2)

   coordinates of grid have the origin in the upper leftmost corner,
   numbers increase going down and to the right
   position (4,2) is 5th column and 3rd row

*/

icon2symbolic(IconicInitialState,
	      problem(SymbolicInitialState,
		      [vehicle(1, pos(4,2))])) :-
	(length(IconicInitialState, 36) ->
	    icon2symbolicInitialState(IconicInitialState, [ ], SymbolicInitialState)
	;
	    format('~nRushHour puzzles should have 36 entries, this does not~n', [ ])).

icon2symbolicInitialState([ ], UnorderedState, OrderedState) :-
	list_to_ord_set([puzzleDimensions(6,6) | UnorderedState], OrderedState).
icon2symbolicInitialState([0 | Rest], TempState, FinalState) :-
	icon2symbolicInitialState(Rest, TempState, FinalState).
icon2symbolicInitialState([Id | Rest], TempState, FinalState) :-
	Id > 0, %% there is a vehicle here
	(member(vehicle(Id, _), TempState) ->
	    NewTemp = TempState %% vehicle already processed, nothing to add
	;   %% new vehicle id, add its info
	    getOrientationAndLength(Id, Rest, Orientation, Length),
	    getPosition(Rest, Pos),
	    NewTemp = [vehicle(Id, Pos), orientation(Id, Orientation), length(Id, Length) | TempState]
	    ),
	icon2symbolicInitialState(Rest, NewTemp, FinalState).

getOrientationAndLength(Id, RestIconic, Orientation, Length) :-
	(nth1(1, RestIconic, Id) ->
	    Orientation = horizontal,
	    (nth1(2, RestIconic, Id) ->
		Length = 3
	    ;
		Length = 2)
	;
	    Orientation = vertical,
	    (nth1(12, RestIconic, Id) ->
		Length = 3
	    ;
		Length = 2)).

getPosition(Rest, pos(X,Y)) :-
	length(Rest, LengthLeft),
	ArrayIndex is 35 - LengthLeft,
	Y is ArrayIndex // 6,
	X is ArrayIndex mod 6.