%	$Id: ontology.pl,v 1.4 2008/10/30 01:34:00 mbar098 Exp mbar098 $

/*  predicates can only be 1 of the following:
- derived
- static
- metaLevel
- fluent

All the metaLevel, derived, static,  and fluent
predicates are described below.

The Ontology:

Name                      Type           Meaning
======================  =========  ============================================ 
vehicle(Id, Pos)       fluent     the upper leftmost location of vehicle Id is Pos

orientation(Id, Direction) static  vehicle Id is vertical or horizontal

length(Id, Length)      static     length of vehicle Id in squares

puzzleDimensions(MaxX, MaxY)  static  max X value inside puzzle is MaxX - 1
                                      max Y value inside puzzle is MaxY - 1

assign(Var, Expression)     metaLevel  Var has value of evaluated Expression

less(Exp1, Exp2)        metaLevel  evaluated Exp1 < evaluated Exp2

geq(Exp1, Exp2)         metaLevel  evaluated Exp1 >= evaluated Exp2

inPuzzle(Pos)           derived    Pos is inside the puzzle

clear(Pos)              derived    no vehicle is occupies Pos

pos(X, Y)               data       coordinate, where upper left-hand corner is 0,0
                                   X increases going to right, Y increases going down

*/

/*
derived predicates:
====================
*/
%% derived predicates info
:- multifile derived/1.
:- dynamic derived/1.
:- multifile inPuzzle/1.
:- dynamic inPuzzle/1.
:- multifile clear/1.
:- dynamic clear/1.
:- multifile verticalVehicleAt/1.
:- dynamic verticalVehicleAt/1.
:- multifile horizontalTruckAt/1.
:- dynamic horizontalTruckAt/1.
:- multifile horizontalVehicleAt/1. 
:- dynamic horizontalVehicleAt/1.
:- multifile verticalTruckAt/1.
:- dynamic verticalTruckAt/1.

derived(inPuzzle/1).
derived(clear/1).
derived(verticalVehicleAt/1).
derived(horizontalTruckAt/1).
derived(horizontalVehicleAt/1).
derived(verticalTruckAt/1).


inPuzzle(pos(X,Y)) :-
	puzzleDimensions(MaxX, MaxY),
	geq(X, 0),
	less(X, MaxX),
	geq(Y,  0),
	less(Y, MaxY).

verticalVehicleAt(Pos) :-
	vehicle(V, Pos),
	orientation(V, vertical).
verticalTruckAt(Pos) :-
	vehicle(T, Pos),
	orientation(T, vertical),
	length(T, 3).
horizontalVehicleAt(Pos) :-
	vehicle(T, Pos),
	orientation(T, horizontal).
horizontalTruckAt(Pos) :-
	vehicle(T, Pos),
	orientation(T, horizontal),
	length(T, 3).
clear(pos(X, Y)) :-
	%% no vehicle positioned on X,Y
	not(vehicle(_, pos(X,Y))),
	%% no vehicle positioned on X, Y - 1 and vertical
	assign(Up1, Y - 1),
	not(verticalVehicleAt(pos(X, Up1))),
	%% no vehicle positioned on X - 1, Y and vertical
	assign(Left1, X - 1),
	not(horizontalVehicleAt(pos(Left1, Y))),
	%% no truck positioned on X, Y - 2 and vertical
	assign(Up2, Y -2),
	not(verticalTruckAt(pos(X, Up2))),
	%% no truck positioned on X - 2, Y and horizontal
	assign(Left2, X - 2),
	not(horizontalTruckAt(pos(Left2, Y))).
	

%% primitive predicate info
:- multifile primitive/1.
:- dynamic primitive/1.
primitive(Pred/Arity) :- not(derived(Pred/Arity)).



/*
static predicates:
==================


*/
%% static predicate info
:- multifile static/1.
:- dynamic static/1.

static(orientation/2).
static(length/2).
static(puzzleDimensions/2).

%% fluent predicates info
:- multifile fluent/1.
:- dynamic fluent/1.
fluent(Pred/Arity) :- not(static(Pred/Arity)).

/*
  metaLevel predicates:
  =====================

*/


%% metaLevel predicates info
:- multifile metaLevel/1.
:- dynamic metaLevel/1.
:- multifile assign/2.
:- dynamic assign/2.
metaLevel(assign/2).
:- multifile less/2.
:- dynamic less/2.
metaLevel(less/2).
:- multifile geq/2.
:- dynamic geq/2.
metaLevel(geq/2).

assign(X, Y) :- X is Y.

less(X, Y) :- X < Y.

geq(X, Y) :- X >= Y.

%% objectLevel predicates info
:- multifile objectLevel/1.
:- dynamic objectLevel/1.
objectLevel(Pred/Arity) :- not(metaLevel(Pred/Arity)).
